/*
 * Copyright (c) 2019-2020 Meltytech, LLC
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.1
import QtQuick.Dialogs 1.1
import QtQuick.Controls 1.5
import QtQuick.Controls.Styles 1.1
import QtQuick.Layouts 1.0
import Shotcut.Controls 1.0

Item {
    width: 550 //600
    height: 450
    property bool blockUpdate: true
    property double startValueRadius: 0
    property double middleValueRadius: 0
    property double endValueRadius: 0
    
    property double startValueMargin: 0
    property double middleValueMargin: 0
    property double endValueMargin: 0
    
    property double startValueInterlettre: 100
    property double middleValueInterlettre: 100
    property double endValueInterlettre: 100
    
    property double startValueAjust: 0
    property double middleValueAjust: 0
    property double endValueAjust: 0
    
    property double startValueBorder: 0
    property double middleValueBorder: 0
    property double endValueBorder: 0
    
    property double startValueFontsize: 32
    property double middleValueFontsize: 32
    property double endValueFontsize: 32
    
    property double startValueFillopacity: 50
    property double middleValueFillopacity: 50
    property double endValueFillopacity: 50
    
    property double startValueBorderopacity: 100
    property double middleValueBorderopacity: 100
    property double endValueBorderopacity: 100
    
    property double startValueFontopacity: 100
    property double middleValueFontopacity: 100
    property double endValueFontopacity: 100
    
    property double startValueRotate: 0
    property double middleValueRotate: 0
    property double endValueRotate: 0
    
    property string startValueRect: '_shotcut:startValue'
    property string middleValueRect: '_shotcut:middleValue'
    property string endValueRect:  '_shotcut:endValue'
    property string rectProperty: 'rect'
    property rect filterRect
    
    Component.onCompleted: {
        filter.blockSignals = true
        filter.set('resource', filter.path + 'filter-text.html')
        filter.set('mlt_resolution_scale', 1)
        filter.set(startValueRect, Qt.rect(profile.width*0.3, profile.height*0.45, profile.width*0.4, profile.height*0.1))
        filter.set(middleValueRect, Qt.rect(profile.width*0.3, profile.height*0.45, profile.width*0.4, profile.height*0.1))
        filter.set(endValueRect, Qt.rect(profile.width*0.3, profile.height*0.45, profile.width*0.4, profile.height*0.1))
        if (filter.isNew) {
            filter.set('texte', 'Type your text here')
            filter.set('mode', 0)
            filter.set('_booster', '0')
            filter.set('_vuivisible', '0')
            filter.set('_onglet', 0)
            filter.set('_ratio', 0)
            filter.set('_halign', 2)
            filter.set('_valign', 2)
            // Police
            filter.set('family', 'Verdana')
            filter.set('fontsize', 32)
            filter.set('bold', 'normal')
            filter.set('italic', 'normal')
            filter.set('smallcaps', 'normal')
            filter.set('textalign', 'center')
            filter.set('fontcolor', '#FFFFFFFF')
            filter.set('fontopacity', 100)
            filter.set('bords', 0)
            filter.set('bordscolor', '#FF000000')
            filter.set('shadow', 0)
            filter.set('blur', 0)
            filter.set('shadowcolor', '#FF000000')
            // Rectangle encadrement
            filter.set(rectProperty, '30%/45%:40%x10%')
            filter.set('radius', 0)
            filter.set('fillcolor', '#FF000000')
            filter.set('fillopacity', 0)
            filter.set('border', 0)
            filter.set('propor', 0)
            filter.set('number', 15)
            filter.set('bordercolor', '#FFFFFFFF')
            filter.set('borderopacity', 100)
            filter.set('rotate', 0)
            filter.set('margin', 0)
            filter.set('ajust', 0)
            filter.set('textajust', 0)
            filter.set('interlettre', 100)
            filter.set('letterwidth', 0)
            filter.set('interligne', 1)
            filter.set('borderover', 0)
            // Fond de page
            filter.set('color', '#00000000')
            filter.set('transparent', 0)
        } 
        else {
            middleValueRadius = filter.getDouble('radius', filter.animateIn)
            middleValueMargin = filter.getDouble('margin', filter.animateIn)
            middleValueInterlettre = filter.getDouble('interlettre', filter.animateIn)
            middleValueAjust = filter.getDouble('ajust', filter.animateIn)
            middleValueBorder = filter.getDouble('border', filter.animateIn)
            middleValueFillopacity = filter.getDouble('fillopacity', filter.animateIn)
            middleValueBorderopacity = filter.getDouble('borderopacity', filter.animateIn)
            middleValueFontopacity = filter.getDouble('fontopacity', filter.animateIn)
            middleValueFontsize = filter.getDouble('fontsize', filter.animateIn)
            middleValueRotate = filter.getDouble('rotate', filter.animateIn)
            filter.set(middleValueRect, filter.getRect(rectProperty, filter.animateIn + 1))
            if (filter.animateIn > 0) {
                startValueRadius = filter.getDouble('radius', 0)
                startValueMargin = filter.getDouble('margin', 0)
                startValueInterlettre = filter.getDouble('interlettre', 0)
                startValueAjust = filter.getDouble('ajust', 0)
                startValueBorder = filter.getDouble('border', 0)
                startValueFillopacity = filter.getDouble('fillopacity', 0)
                startValueBorderopacity = filter.getDouble('borderopacity', 0)
                startValueFontopacity = filter.getDouble('fontopacity', 0)
                startValueFontsize = filter.getDouble('fontsize', 0)
                startValueRotate = filter.getDouble('rotate', 0)
                filter.set(startValueRect, filter.getRect(rectProperty, 0))
            }
            if (filter.animateOut > 0) {
                endValueRadius = filter.getDouble('radius', filter.duration - 1)
                endValueMargin = filter.getDouble('margin', filter.duration - 1)
                endValueInterlettre = filter.getDouble('interlettre', filter.duration - 1)
                endValueAjust = filter.getDouble('ajust', filter.duration - 1)
                endValueBorder = filter.getDouble('border', filter.duration - 1)
                endValueFillopacity = filter.getDouble('fillopacity', filter.duration - 1)
                endValueBorderopacity = filter.getDouble('borderopacity', filter.duration - 1)
                endValueFontopacity = filter.getDouble('fontopacity', filter.duration - 1)
                endValueFontsize = filter.getDouble('fontsize', filter.duration - 1)
                endValueRotate = filter.getDouble('rotate', filter.duration - 1)
                filter.set(endValueRect, filter.getRect(rectProperty, filter.duration - 1))
            }
        }
        filter.blockSignals = false
        if (filter.get('_booster') == '1') booster.checked = false
        else booster.checked = true
        if (filter.get('_vuivisible') == '1') vuivisible.checked = false
        else vuivisible.checked = true
        ctext.text = filter.get('texte')
        if (filter.get('mode') == 1) modeTitreRadioButton.checked = true
        else modeStdRadioButton.checked = true
        tabview.currentIndex = (filter.get('_onglet') == "")? 1 : filter.get('_onglet')
        bgcolorSwatch.value = filter.get('color')
        refclip.checked = (filter.get('transparent') == 1)? true : false
        // ********* Onglet Police *********
        boldid.checked = (filter.get('bold') == 'bold')? true : false
        italicid.checked = (filter.get('italic') == 'italic')? true : false
        smallcapsid.checked = (filter.get('smallcaps') == 'small-caps')? true : false
        if (filter.get('textalign') == '') filter.set('textalign', 'center')
        textaligncenterid.checked = (filter.get('textalign') == 'center')? true : false
        textalignleftid.checked = (filter.get('textalign') == 'left')? true : false
        textalignrightid.checked = (filter.get('textalign') == 'right')? true : false
        fontButton.text = filter.get('family')
        //fontRadioButton.checked = true
        fontcolorSwatch.value = filter.get('fontcolor')
        bordscolorSwatch.value = filter.get('bordscolor')
        shadowcolorSwatch.value = filter.get('shadowcolor')
        bords.value = filter.get('bords')
        shadow.value = filter.get('shadow')
        blur.value = filter.get('blur')
        // ********* Onglet Encadrement *********
        if (filter.get('propor') == 1) {
            proportionid.checked = true
            contour.suffix = ' %'
        } 
        else {
            proportionid.checked = false
            contour.suffix = ' px'
        }
        nombreid.currentIndex = filter.get('number')
        //mastercolorSwatch.value = fillcolorSwatch.value = filter.get('fillcolor')
        //bordercolorSwatch.value = filter.get('bordercolor')
        interligneid.value = filter.get('interligne')
        textajustid.checked = filter.get('textajust') == 1
        letterwidthid.checked = filter.get('letterwidth') == 1
        borderoverid.currentIndex = (filter.get('borderover') == 1)? 1 : 0
        
        centerRadioButton.checked = true
        leftRadioButton.checked = filter.get('_halign') == 1
        rightRadioButton.checked = filter.get('_halign') == 3
        middleRadioButton.checked = true
        topRadioButton.checked = filter.get('_valign') == 1
        bottomRadioButton.checked = filter.get('_valign') == 3
        
        fillRadioButton.checked = true
        setRadiusControls()
        setMarginControls()
        setInterlettreControls()
        setAjustControls()
        setBorderControls()
        setFillopacityControls()
        setBorderopacityControls()
        setFontopacityControls()
        setFontsizeControls()
        setRotateControls()
        setRectControls()
        mastercolorSwatch.value = fillcolorSwatch.value = setOpacity(filter.get('fillcolor'), fillopacityid.value)
        bordercolorSwatch.value = setOpacity(filter.get('bordercolor'), borderopacityid.value)
        ratioCheckbox.checked = (filter.get('_ratio') == 0 || filter.keyframeCount(rectProperty) > 0)? false : true
                
        if (filter.isNew) {
            filter.set(rectProperty, filter.getRect(rectProperty))
        }
    }
    
    function getPosition() {
        return Math.max(producer.position - (filter.in - producer.in), 0)
    }
    
    function setRadiusControls() {
        var position = getPosition()
        blockUpdate = true
        radiusid.value = filter.getDouble('radius', position) * radiusid.maximumValue
        blockUpdate = false
        radiusid.enabled = position <= 0 || (position >= (filter.animateIn - 1) && position <= (filter.duration - filter.animateOut)) || position >= (filter.duration - 1)
        radiusKeyframesButton.checked = filter.keyframeCount('radius') > 0 && filter.animateIn <= 0 && filter.animateOut <= 0
    }
    
    function setMarginControls() {
        var position = getPosition()
        blockUpdate = true
        marginid.value = filter.getDouble('margin', position)
        blockUpdate = false
        marginid.enabled = position <= 0 || (position >= (filter.animateIn - 1) && position <= (filter.duration - filter.animateOut)) || position >= (filter.duration - 1)
        marginKeyframesButton.checked = filter.keyframeCount('margin') > 0 && filter.animateIn <= 0 && filter.animateOut <= 0
    }
    
    function setInterlettreControls() {
        var position = getPosition()
        blockUpdate = true
        interlettreid.value = filter.getDouble('interlettre', position)
        blockUpdate = false
        interlettreid.enabled = position <= 0 || (position >= (filter.animateIn - 1) && position <= (filter.duration - filter.animateOut)) || position >= (filter.duration - 1)
        interlettreKeyframesButton.checked = filter.keyframeCount('interlettre') > 0 && filter.animateIn <= 0 && filter.animateOut <= 0
    }
    
    function setAjustControls() {
        var position = getPosition()
        blockUpdate = true
        ajustid.value = filter.getDouble('ajust', position)
        blockUpdate = false
        ajustid.enabled = position <= 0 || (position >= (filter.animateIn - 1) && position <= (filter.duration - filter.animateOut)) || position >= (filter.duration - 1)
        ajustKeyframesButton.checked = filter.keyframeCount('ajust') > 0 && filter.animateIn <= 0 && filter.animateOut <= 0
    }
    
    function setBorderControls() {
        var position = getPosition()
        blockUpdate = true
        contour.value = filter.getDouble('border', position)
        blockUpdate = false
        contour.enabled = position <= 0 || (position >= (filter.animateIn - 1) && position <= (filter.duration - filter.animateOut)) || position >= (filter.duration - 1)
        borderKeyframesButton.checked = filter.keyframeCount('border') > 0 && filter.animateIn <= 0 && filter.animateOut <= 0
    }
    
    function setFillopacityControls() {
        var position = getPosition()
        blockUpdate = true
        fillopacityid.value = filter.getDouble('fillopacity', position)
        blockUpdate = false
        fillopacityid.enabled = position <= 0 || (position >= (filter.animateIn - 1) && position <= (filter.duration - filter.animateOut)) || position >= (filter.duration - 1)
        fillopacityKeyframesButton.checked = filter.keyframeCount('fillopacity') > 0 && filter.animateIn <= 0 && filter.animateOut <= 0
    }

    function setBorderopacityControls() {
        var position = getPosition()
        blockUpdate = true
        borderopacityid.value = filter.getDouble('borderopacity', position)
        blockUpdate = false
        borderopacityid.enabled = position <= 0 || (position >= (filter.animateIn - 1) && position <= (filter.duration - filter.animateOut)) || position >= (filter.duration - 1)
        borderopacityKeyframesButton.checked = filter.keyframeCount('borderopacity') > 0 && filter.animateIn <= 0 && filter.animateOut <= 0
    }

    function setFontopacityControls() {
        var position = getPosition()
        blockUpdate = true
        fontopacityid.value = filter.getDouble('fontopacity', position)
        blockUpdate = false
        fontopacityid.enabled = position <= 0 || (position >= (filter.animateIn - 1) && position <= (filter.duration - filter.animateOut)) || position >= (filter.duration - 1)
        fontopacityKeyframesButton.checked = filter.keyframeCount('fontopacity') > 0 && filter.animateIn <= 0 && filter.animateOut <= 0
    }

    function setFontsizeControls() {
        var position = getPosition()
        blockUpdate = true
        fontsizeid.value = filter.getDouble('fontsize', position)
        blockUpdate = false
        fontsizeid.enabled = position <= 0 || (position >= (filter.animateIn - 1) && position <= (filter.duration - filter.animateOut)) || position >= (filter.duration - 1)
        fontsizeKeyframesButton.checked = filter.keyframeCount('fontsize') > 0 && filter.animateIn <= 0 && filter.animateOut <= 0
    }

    function setRotateControls() {
        var position = getPosition()
        blockUpdate = true
        rotateslider.value = filter.getDouble('rotate', position)
        blockUpdate = false
        rotateslider.enabled = position <= 0 || (position >= (filter.animateIn - 1) && position <= (filter.duration - filter.animateOut)) || position >= (filter.duration - 1)
        rotateKeyframesButton.checked = filter.keyframeCount('rotate') > 0 && filter.animateIn <= 0 && filter.animateOut <= 0
    }

    function setRectControls() {
        var position = getPosition()
        var newValue = filter.getRect(rectProperty, position)
        if (filterRect !== newValue) {
            filterRect = newValue
            rectX.value = filterRect.x.toFixed()
            rectY.value = filterRect.y.toFixed()
            rectW.value = filterRect.width.toFixed()
            rectH.value = filterRect.height.toFixed()
        }
        var enabled = position <= 0 || (position >= (filter.animateIn - 1) && position <= (filter.duration - filter.animateOut)) || position >= (filter.duration - 1)
        rectX.enabled = enabled
        rectY.enabled = enabled
        rectW.enabled = enabled
        rectH.enabled = enabled
    }
    
    function updateFilterRadius(position) {
        if (blockUpdate) return
        var value = radiusid.value / 100.0

        if (position !== null) {
            if (position <= 0 && filter.animateIn > 0) {
                startValueRadius = value
            } else if (position >= filter.duration - 1 && filter.animateOut > 0) {
                endValueRadius = value
            } else {
                middleValueRadius = value
            }
        }

        if (filter.animateIn > 0 || filter.animateOut > 0) {
            filter.resetProperty('radius')
            radiusKeyframesButton.checked = false
            if (filter.animateIn > 0) {
                filter.set('radius', startValueRadius, 0)
                filter.set('radius', middleValueRadius, filter.animateIn - 1)
            }
            if (filter.animateOut > 0) {
                filter.set('radius', middleValueRadius, filter.duration - filter.animateOut)
                filter.set('radius', endValueRadius, filter.duration - 1)
            }
        } else if (!radiusKeyframesButton.checked) {
            filter.resetProperty('radius')
            filter.set('radius', middleValueRadius)
        } else if (position !== null) {
            filter.set('radius', value, position)
        }
    }
    
    function updateFilterMargin(position) {
        if (blockUpdate) return
        var value = marginid.value

        if (position !== null) {
            if (position <= 0 && filter.animateIn > 0) {
                startValueMargin = value
            } else if (position >= filter.duration - 1 && filter.animateOut > 0) {
                endValueMargin = value
            } else {
                middleValueMargin = value
            }
        }

        if (filter.animateIn > 0 || filter.animateOut > 0) {
            filter.resetProperty('margin')
            marginKeyframesButton.checked = false
            if (filter.animateIn > 0) {
                filter.set('margin', startValueMargin, 0)
                filter.set('margin', middleValueMargin, filter.animateIn - 1)
            }
            if (filter.animateOut > 0) {
                filter.set('margin', middleValueMargin, filter.duration - filter.animateOut)
                filter.set('margin', endValueMargin, filter.duration - 1)
            }
        } else if (!marginKeyframesButton.checked) {
            filter.resetProperty('margin')
            filter.set('margin', middleValueMargin)
        } else if (position !== null) {
            filter.set('margin', value, position)
        }
    }
    
    function updateFilterInterlettre(position) {
        if (blockUpdate) return
        var value = interlettreid.value

        if (position !== null) {
            if (position <= 0 && filter.animateIn > 0) {
                startValueInterlettre = value
            } else if (position >= filter.duration - 1 && filter.animateOut > 0) {
                endValueInterlettre = value
            } else {
                middleValueInterlettre = value
            }
        }

        if (filter.animateIn > 0 || filter.animateOut > 0) {
            filter.resetProperty('interlettre')
            interlettreKeyframesButton.checked = false
            if (filter.animateIn > 0) {
                filter.set('interlettre', startValueInterlettre, 0)
                filter.set('interlettre', middleValueInterlettre, filter.animateIn - 1)
            }
            if (filter.animateOut > 0) {
                filter.set('interlettre', middleValueInterlettre, filter.duration - filter.animateOut)
                filter.set('interlettre', endValueInterlettre, filter.duration - 1)
            }
        } else if (!interlettreKeyframesButton.checked) {
            filter.resetProperty('interlettre')
            filter.set('interlettre', middleValueInterlettre)
        } else if (position !== null) {
            filter.set('interlettre', value, position)
        }
    }
    
    function updateFilterAjust(position) {
        if (blockUpdate) return
        var value = ajustid.value

        if (position !== null) {
            if (position <= 0 && filter.animateIn > 0) {
                startValueAjust = value
            } else if (position >= filter.duration - 1 && filter.animateOut > 0) {
                endValueAjust = value
            } else {
                middleValueAjust = value
            }
        }

        if (filter.animateIn > 0 || filter.animateOut > 0) {
            filter.resetProperty('ajust')
            ajustKeyframesButton.checked = false
            if (filter.animateIn > 0) {
                filter.set('ajust', startValueAjust, 0)
                filter.set('ajust', middleValueAjust, filter.animateIn - 1)
            }
            if (filter.animateOut > 0) {
                filter.set('ajust', middleValueAjust, filter.duration - filter.animateOut)
                filter.set('ajust', endValueAjust, filter.duration - 1)
            }
        } else if (!ajustKeyframesButton.checked) {
            filter.resetProperty('ajust')
            filter.set('ajust', middleValueAjust)
        } else if (position !== null) {
            filter.set('ajust', value, position)
        }
    }
    
    function updateFilterBorder(position) {
        if (blockUpdate) return
        var value = contour.value

        if (position !== null) {
            if (position <= 0 && filter.animateIn > 0) {
                startValueBorder = value
            } else if (position >= filter.duration - 1 && filter.animateOut > 0) {
                endValueBorder = value
            } else {
                middleValueBorder = value
            }
        }

        if (filter.animateIn > 0 || filter.animateOut > 0) {
            filter.resetProperty('border')
            borderKeyframesButton.checked = false
            if (filter.animateIn > 0) {
                filter.set('border', startValueBorder, 0)
                filter.set('border', middleValueBorder, filter.animateIn - 1)
            }
            if (filter.animateOut > 0) {
                filter.set('border', middleValueBorder, filter.duration - filter.animateOut)
                filter.set('border', endValueBorder, filter.duration - 1)
            }
        } else if (!borderKeyframesButton.checked) {
            filter.resetProperty('border')
            filter.set('border', middleValueBorder)
        } else if (position !== null) {
            filter.set('border', value, position)
        }
    }
    
    function updateFilterFillopacity(position) {
        if (blockUpdate) return
        var value = fillopacityid.value

        if (position !== null) {
            if (position <= 0 && filter.animateIn > 0) {
                startValueFillopacity = value
            } else if (position >= filter.duration - 1 && filter.animateOut > 0) {
                endValueFillopacity = value
            } else {
                middleValueFillopacity = value
            }
        }

        if (filter.animateIn > 0 || filter.animateOut > 0) {
            filter.resetProperty('fillopacity')
            fillopacityKeyframesButton.checked = false
            if (filter.animateIn > 0) {
                filter.set('fillopacity', startValueFillopacity, 0)
                filter.set('fillopacity', middleValueFillopacity, filter.animateIn - 1)
            }
            if (filter.animateOut > 0) {
                filter.set('fillopacity', middleValueFillopacity, filter.duration - filter.animateOut)
                filter.set('fillopacity', endValueFillopacity, filter.duration - 1)
            }
        } else if (!fillopacityKeyframesButton.checked) {
            filter.resetProperty('fillopacity')
            filter.set('fillopacity', middleValueFillopacity)
        } else if (position !== null) {
            filter.set('fillopacity', value, position)
        }
    }

    function updateFilterBorderopacity(position) {
        if (blockUpdate) return
        var value = borderopacityid.value

        if (position !== null) {
            if (position <= 0 && filter.animateIn > 0) {
                startValueBorderopacity = value
            } else if (position >= filter.duration - 1 && filter.animateOut > 0) {
                endValueBorderopacity = value
            } else {
                middleValueBorderopacity = value
            }
        }

        if (filter.animateIn > 0 || filter.animateOut > 0) {
            filter.resetProperty('borderopacity')
            borderopacityKeyframesButton.checked = false
            if (filter.animateIn > 0) {
                filter.set('borderopacity', startValueBorderopacity, 0)
                filter.set('borderopacity', middleValueBorderopacity, filter.animateIn - 1)
            }
            if (filter.animateOut > 0) {
                filter.set('borderopacity', middleValueBorderopacity, filter.duration - filter.animateOut)
                filter.set('borderopacity', endValueBorderopacity, filter.duration - 1)
            }
        } else if (!borderopacityKeyframesButton.checked) {
            filter.resetProperty('borderopacity')
            filter.set('borderopacity', middleValueBorderopacity)
        } else if (position !== null) {
            filter.set('borderopacity', value, position)
        }
    }

    function updateFilterFontopacity(position) {
        if (blockUpdate) return
        var value = fontopacityid.value

        if (position !== null) {
            if (position <= 0 && filter.animateIn > 0) {
                startValueFontopacity = value
            } else if (position >= filter.duration - 1 && filter.animateOut > 0) {
                endValueFontopacity = value
            } else {
                middleValueFontopacity = value
            }
        }

        if (filter.animateIn > 0 || filter.animateOut > 0) {
            filter.resetProperty('fontopacity')
            fontopacityKeyframesButton.checked = false
            if (filter.animateIn > 0) {
                filter.set('fontopacity', startValueFontopacity, 0)
                filter.set('fontopacity', middleValueFontopacity, filter.animateIn - 1)
            }
            if (filter.animateOut > 0) {
                filter.set('fontopacity', middleValueFontopacity, filter.duration - filter.animateOut)
                filter.set('fontopacity', endValueFontopacity, filter.duration - 1)
            }
        } else if (!fontopacityKeyframesButton.checked) {
            filter.resetProperty('fontopacity')
            filter.set('fontopacity', middleValueFontopacity)
        } else if (position !== null) {
            filter.set('fontopacity', value, position)
        }
    }

    function updateFilterFontsize(position) {
        if (blockUpdate) return
        var value = fontsizeid.value

        if (position !== null) {
            if (position <= 0 && filter.animateIn > 0) {
                startValueFontsize = value
            } else if (position >= filter.duration - 1 && filter.animateOut > 0) {
                endValueFontsize = value
            } else {
                middleValueFontsize = value
            }
        }

        if (filter.animateIn > 0 || filter.animateOut > 0) {
            filter.resetProperty('fontsize')
            fontsizeKeyframesButton.checked = false
            if (filter.animateIn > 0) {
                filter.set('fontsize', startValueFontsize, 0)
                filter.set('fontsize', middleValueFontsize, filter.animateIn - 1)
            }
            if (filter.animateOut > 0) {
                filter.set('fontsize', middleValueFontsize, filter.duration - filter.animateOut)
                filter.set('fontsize', endValueFontsize, filter.duration - 1)
            }
        } else if (!fontsizeKeyframesButton.checked) {
            filter.resetProperty('fontsize')
            filter.set('fontsize', middleValueFontsize)
        } else if (position !== null) {
            filter.set('fontsize', value, position)
        }
    }

    function updateFilterRotate(position) {
        if (blockUpdate) return
        var value = rotateslider.value

        if (position !== null) {
            if (position <= 0 && filter.animateIn > 0) {
                startValueRotate = value
            } else if (position >= filter.duration - 1 && filter.animateOut > 0) {
                endValueRotate = value
            } else {
                middleValueRotate = value
            }
        }

        if (filter.animateIn > 0 || filter.animateOut > 0) {
            filter.resetProperty('rotate')
            rotateKeyframesButton.checked = false
            if (filter.animateIn > 0) {
                filter.set('rotate', startValueRotate, 0)
                filter.set('rotate', middleValueRotate, filter.animateIn - 1)
            }
            if (filter.animateOut > 0) {
                filter.set('rotate', middleValueRotate, filter.duration - filter.animateOut)
                filter.set('rotate', endValueRotate, filter.duration - 1)
            }
        } else if (!rotateKeyframesButton.checked) {
            filter.resetProperty('rotate')
            filter.set('rotate', middleValueRotate)
        } else if (position !== null) {
            filter.set('rotate', value, position)
        }
    }

    function updateFilterRect(position) {
        var rect

        if (position !== null) {
            filter.blockSignals = true
            if (position <= 0 && filter.animateIn > 0) {
                filter.set(startValueRect, filterRect)
            } else if (position >= filter.duration - 1 && filter.animateOut > 0) {
                filter.set(endValueRect, filterRect)
            } else {
                filter.set(middleValueRect, filterRect)
            }
            filter.blockSignals = false
        }

        if (filter.animateIn > 0 || filter.animateOut > 0) {
            filter.resetProperty(rectProperty)
            positionKeyframesButton.checked = false
            if (filter.animateIn > 0) {
                rect = filter.getRect(startValueRect)
                filter.set(rectProperty, rect, 1.0, 0)
                rect = filter.getRect(middleValueRect)
                filter.set(rectProperty, rect, 1.0, filter.animateIn - 1)
            }
            if (filter.animateOut > 0) {
                rect = filter.getRect(middleValueRect)
                filter.set(rectProperty, rect, 1.0, filter.duration - filter.animateOut)
                rect = filter.getRect(endValueRect)
                filter.set(rectProperty, rect, 1.0, filter.duration - 1)
            }
        } else if (!positionKeyframesButton.checked) {
            filter.resetProperty(rectProperty)
            rect = filter.getRect(middleValueRect)
            filter.set(rectProperty, rect)
        } else if (position !== null) {
            filter.set(rectProperty, filterRect, 1.0, position)
        }
    }
    
    function getOpacity(mltColor) {
        var a = parseInt(mltColor.substring(1, 3), 16);
        return Math.round(a/2.55);
    }
    
    function setOpacity(mltColor, opa) {
        var opastr = (mltColor.length > 7)? Math.round(opa*2.55).toString(16) : "FF";
        if ((opastr.length % 2) > 0) {
            opastr = "0" + opastr;
        }
        var suite = (mltColor.length > 7)? mltColor.substring(3, 9) : mltColor.substring(1, 7);
        return '#' + opastr + suite;
    }
    
    ExclusiveGroup { id: halignGroup }
    ExclusiveGroup { id: valignGroup }
    ExclusiveGroup { id: colorGroup }
    ExclusiveGroup { id: textAlignGroup }
    ExclusiveGroup { id: modeGroup }

    GridLayout {
        columns: 2
        anchors.horizontalCenter: parent.horizontalCenter
        //anchors.left: parent.left
        //anchors.leftMargin: 50
        anchors.top: parent.top
        anchors.topMargin: 10
        
        RowLayout {
            Layout.columnSpan: 2
            ToolButton {
                Layout.preferredWidth: 22
                Layout.preferredHeight: 22
                iconName: 'task-attempt'
                iconSource: 'qrc:///icons/oxygen/32x32/status/task-attempt.png'
                tooltip: qsTr('Important notice about WebVfx')
                onClicked: helpDialog.visible = true
                MessageDialog {
                    id: helpDialog
                    visible: false
                    modality: Qt.ApplicationModal
                    icon: StandardIcon.Information
        			text: qsTr("<u>Filters using <b>WebVfx</b></u> :<p>"
					+ "Filters that use <b>WebVfx</b> often crash Shotcut when closing a project "
					+ "or quitting the program. Before quitting or closing the project, save "
					+ "it if necessary, then click on another clip (which does not have a "
					+ "WebVfx filter) to select it. You must also move the playhead away "
					+ "from the clip so that it is not displayed in the player.<p>"
					+ "You should then be able to close the project or quit Shotcut without "
					+ "any problems.<br>")
                    standardButtons: StandardButton.Ok
                }
            } // WebVfx
            Label { 
                horizontalAlignment: Text.AlignLeft
                text: 'WebVfx'
            } // WebVfxItem { width: 15 }
            Item { width: 40 }
            Label {text: qsTr('Mode')}
            Item { width: 5 }
            RadioButton {
                id: modeStdRadioButton
                text: '  Standard'
                exclusiveGroup: modeGroup
                onClicked: {
                    if (checked) {
                        filter.set('mode', 0)
                        smallcapsid.visible = false
                        filter.set('smallcaps', 'normal')
                        fontsizeid.value -= 1
                    }
                }
            } // Mode Standard
            Item { width: 5 }
            RadioButton {
                id: modeTitreRadioButton
                text: '  Title'
                exclusiveGroup: modeGroup
                onClicked: {
                    if (checked) {
                        filter.set('mode', 1)
                        smallcapsid.visible = true
                    }
                }
            } // Mode Titre
            Item { Layout.fillWidth: true }
            ToolButton {
                implicitWidth: 26
                implicitHeight: 26
                iconName: 'help-about'
                iconSource: 'qrc:///icons/oxygen/32x32/actions/help-about.png'
                tooltip: qsTr('Learn more about the two modes.')
                onClicked: modeDialog.visible = true
                MessageDialog {
                    id: modeDialog
                    visible: false
                    modality: Qt.ApplicationModal
                    icon: StandardIcon.Information
        			text: qsTr(""
					+ "Although the <b>Text: WebVfx</b> filter is capable of displaying "
					+ "paragraphs of text, it is intended mainly for creating and "
					+ "animating short texts, like titles or lower thirds.<p>"
					+ "<b>Standard mode.</b><br>"
					+ "This is the default mode of this filter. In this mode, the "
					+ "text is processed line by line. The text can be constrained "
					+ "to stay within the limits of the Frame and you can adjust "
					+ "the length of the lines by narrowing the characters and spaces.<br>"
					+ "This allows for example to obtain a justified multi-line text.<br>"
					+ "The lines can therefore be narrowed but not lengthened.<p>"
					+ "<b>Title mode.</b><br>"
					+ "In this mode, the text is processed character by character. It "
					+ "can no longer be constrained to stay within the limits of the "
					+ "Frame, but the length of the lines can still be manipulated "
					+ "by changing the space between each character and the width "
					+ "of the characters.<br>The lines of text can therefore be made "
					+ "shorter or longer.<br>")
                    standardButtons: StandardButton.Ok
                }
            } // Help
            Item { width: 20 }
            Label {text: 'VUI'}
            ToggleButton {
                id: vuivisible
                implicitWidth: 26
                implicitHeight: 20
                iconName: checked ? 'layer-visible-on' : 'layer-visible-off'
                iconSource: checked ? 'qrc:///icons/oxygen/32x32/actions/layer-visible-on.png' : 'qrc:///icons/oxygen/32x32/actions/layer-visible-off.png'
                tooltip: qsTr('Display/Hide the bounding box of the visual user interface (VUI)')
                onClicked: {
                    if (checked) filter.set('_vuivisible', '0')
                    else filter.set('_vuivisible', '1')
                }
            } // VUI
            //Item { width: 10 }
            
        } // Ligne supérieure
        
        TabView {
            id: tabview
            Layout.columnSpan: 2
            Layout.preferredWidth: 500 //480
            Layout.preferredHeight: {
                (tabview.currentIndex == 0)? 340 : 
                (tabview.currentIndex == 1)? 340 : 
                (tabview.currentIndex == 2)? 340 : 210
            }
            style: TabViewStyle {
                frameOverlap: 1
                tab: Rectangle {
                    color: styleData.selected ? "#434343" :"#363636"
                    border.color:  styleData.selected ? "#434343" : "#111111"
                    implicitWidth: Math.min(text.width + 27, 150)
                    implicitHeight: 30
                    radius: 2
                    Text {
                        id: text
                        anchors.centerIn: parent
                        text: styleData.title
                        color: "#eeeeee"
                    }
                }
                frame: Rectangle { color: "#434343" }
            }
            Tab {
                title: "Text / Font"
                active: true
                onVisibleChanged: if (tabview.currentIndex == 0) {
                    filter.set('_onglet', 0)
                    fontRadioButton.checked = true
                    mastercolorSwatch.value = fontcolorSwatch.value
                }
            }
            Tab {
                title: "Frame"
                active: true
                onVisibleChanged: if (tabview.currentIndex == 1) {
                    filter.set('_onglet', 1)
                    if (borderRadioButton.checked) {
                        mastercolorSwatch.value = bordercolorSwatch.value
                    }
                    else {
                        fillRadioButton.checked = true
                        mastercolorSwatch.value = fillcolorSwatch.value
                    }
                }
            }
            Tab {
                title: "Text formatting"
                active: true
                onVisibleChanged: if (tabview.currentIndex == 2) {
                    filter.set('_onglet', 2)
                    bgRadioButton.checked = true
                    mastercolorSwatch.value = bgcolorSwatch.value
                }
            }
        } // Dessin des onglets
        
        GridLayout {
            id: text
            Layout.columnSpan: 2
            columns: 6
            anchors.fill: parent
            anchors.leftMargin: 20
            anchors.topMargin: 72
            visible: tabview.currentIndex == 0
            
            Label {
                text: qsTr('Text')
                Layout.alignment: Qt.AlignRight
            }
            RowLayout {
                Layout.columnSpan: 3
                TextArea {
                    id: ctext
                    Layout.preferredHeight: 43
                    Layout.preferredWidth: 265
                    text: ctext
                    property int maxLength: 256
                }
            }
            RowLayout {
                Item { width: 10 }
                Button {
                    implicitWidth : 70
                    text: qsTr('OK')
                    onClicked: {
                        filter.set('texte', ctext.text)
                        ctext.focus = false
                    }
                    ToolTip {
                        text: 'Validate text modification'
                    }
                }
            }
            Item { width: 20 } // Fin Ligne Texte
            
            Item { 
                Layout.columnSpan: 6
                height : 0
                Layout.fillWidth: true
            } // Double l'interligne
            
            Label {
                text: qsTr('Font')
                Layout.alignment: Qt.AlignRight
            }
            RowLayout {
                Layout.columnSpan: 4
                //spacing: 5
                Item { width: 10 }
                Button {
                    id: fontButton
                    Layout.preferredWidth: 300
                    text: filter.get('family')
                    onClicked: {
                    fontDialog.open()
                }
                    FontDialog {
                    id: fontDialog
                    title: "Please choose a font"
                    modality: Qt.ApplicationModal
                    property string fontFamily: ''
                    onFontChanged: {
                        filter.set('family', font.family)
                        fontButton.text = font.family
                    }
                    onAccepted: fontFamily = font.family
                    onRejected: {
                        filter.set('family', fontFamily)
                        fontButton.text = font.family
                    }
                }
                }
            }
            Item { implicitWidth: 20 } // Fin Ligne Police
            
            Label {
                text: qsTr('Style')
                Layout.alignment: Qt.AlignRight
            }
            RowLayout {
                Layout.columnSpan: 2
                Item { width: 10 }
                ToggleButton {
                    id: boldid
                    implicitWidth: 26
                    implicitHeight: 20
                    iconName: 'format-text-bold'
                    iconSource: 'qrc:///icons/oxygen/32x32/actions/format-text-bold.png'
                    tooltip: qsTr('Bold')
                    onClicked: {
                        if (checked) filter.set('bold', 'bold')
                        else filter.set('bold', 'normal')
                    }
                }
                ToggleButton {
                    id: italicid
                    implicitWidth: 26
                    implicitHeight: 20
                    iconName: 'format-text-italic'
                    iconSource: 'qrc:///icons/oxygen/32x32/actions/format-text-italic.png'
                    tooltip: qsTr('Italic')
                    onClicked: {
                        if (checked) filter.set('italic', 'italic')
                        else filter.set('italic', 'normal')
                    }
                }
                ToggleButton {
                    id: smallcapsid
                    visible: filter.get('mode') == 1
                    implicitWidth: 26
                    implicitHeight: 20
                    text: qsTr('Sc       ')
                    tooltip: qsTr('Small-caps')
                    onClicked: {
                        if (checked) {
                            filter.set('smallcaps', 'small-caps')
                            fontsizeid.value += 1
                        }
                        else {
                            filter.set('smallcaps', 'normal')
                            fontsizeid.value -= 1
                        }
                    }
                }
                ToggleButton { // Faux bouton SC désactivé pour pouvoir afficher bulle d'aide
                    visible: smallcapsid.visible == false
                    opacity: 0.5
                    implicitWidth: 26
                    implicitHeight: 20
                    text: qsTr('Sc       ')
                    tooltip: qsTr('Small-caps : Only available in Title mode')
                    onClicked: checked = false
                }
            }
            Label {
                text: qsTr('Size')
                Layout.alignment: Qt.AlignRight
            }
            RowLayout {
                SpinBox {
                    id: fontsizeid
                    implicitWidth : 70
                    decimals: 0
                    minimumValue: 0
                    maximumValue: profile.height
                    suffix: qsTr(' px')
                    stepSize: 1
                    value: 32
                    Keys.onEnterPressed: focus = false
                    Keys.onReturnPressed: focus = false
                    onValueChanged: updateFilterFontsize(getPosition())
                    ToolTip { text: 'Font size' }
                }
                UndoButton {
                    onClicked: fontsizeid.value = 32
                }
            }
            KeyframesButton {
                id: fontsizeKeyframesButton
                Layout.alignment: Qt.AlignCenter
                checked: filter.animateIn <= 0 && filter.animateOut <= 0 && filter.keyframeCount('fontsize') > 0
                onToggled: {
                    var value = fontsizeid.value
                    if (checked) {
                        filter.clearSimpleAnimation('fontsize')
                        filter.set('fontsize', value, getPosition())
                    } else {
                        filter.resetProperty('fontsize')
                        filter.set('fontsize', value)
                    }
                    checked = filter.keyframeCount('fontsize') > 0 && filter.animateIn <= 0 && filter.animateOut <= 0
                }
            } // Fin Ligne Style
                
            Label {
                text: qsTr('Outline')
                Layout.alignment: Qt.AlignRight
            }
            Label {
                text: qsTr('    Thickness')
                Layout.alignment: Qt.AlignRight
            }
            RowLayout {
                Layout.columnSpan: 3
                SpinBox { 
                    id: bords
                    Layout.preferredWidth: 65
                    minimumValue: 0
                    maximumValue: 99
                    decimals: 0
                    suffix: qsTr(' px')
                    Keys.onEnterPressed: focus = false
                    Keys.onReturnPressed: focus = false
                    onValueChanged: filter.set('bords', value)
                }
                UndoButton {
                    onClicked: bords.value = 0
                }
            }
            Item { implicitWidth: 20 } // Fin Ligne Contour Police
            
            Label {
                text: qsTr('Shadow')
                Layout.alignment: Qt.AlignRight
            }
            Label {
                text: qsTr('Offset')
                Layout.alignment: Qt.AlignRight
            }
            RowLayout {
                SpinBox {
                    id: shadow
                    Layout.preferredWidth: 65
                    minimumValue: 0
                    maximumValue: 99
                    decimals: 0
                    suffix: qsTr(' px')
                    Keys.onEnterPressed: focus = false
                    Keys.onReturnPressed: focus = false
                    onValueChanged: filter.set('shadow', value)
                }
                UndoButton {
                    onClicked: shadow.value = 0
                }
            }
            Label {
                text: qsTr('Blur')
                Layout.alignment: Qt.AlignRight
            }
            RowLayout {
                SpinBox { // Flou
                    id: blur
                    implicitWidth : 70
                    minimumValue: 0
                    maximumValue: 99
                    decimals: 0
                    suffix: qsTr(' px')
                    Keys.onEnterPressed: focus = false
                    Keys.onReturnPressed: focus = false
                    onValueChanged: filter.set('blur', value)
                }
                UndoButton {
                    onClicked: blur.value = 0
                }
            }
            Item { width: 20 } // Fin Ligne Ombre
            
            Item { 
                Layout.columnSpan: 6
                height : 0
                Layout.fillWidth: true
            } // Double l'interligne
            
            Rectangle {
                Layout.alignment: Qt.AlignRight
                width: 55
                height: 1
                color: "gray"
            }
            RowLayout {
                Layout.columnSpan: 5
                spacing: 5
                Label {
                    text: 'Color and Opacity'
                    //Layout.alignment: Qt.AlignRight
                }
                Rectangle {
                    color: "gray"
                    Layout.fillWidth: true
                    height: 1
                }
            } // Fin Titre Couleurs
            
            Label {
                text: qsTr('Characters')
                Layout.alignment: Qt.AlignRight
            }
            RowLayout {
                Layout.columnSpan: 4
                RadioButton {
                    id: fontRadioButton
                    exclusiveGroup: colorGroup
                    onClicked: mastercolorSwatch.value = fontcolorSwatch.value
                }
                ColorPicker {
                    id: fontcolorSwatch
                    alpha: true
                    eyedropper: false // donc pas besoin de "filter.set (disable…)"
                    property bool isReady: false
                    Component.onCompleted: isReady = true
                    onValueChanged: {
                        if (isReady) {
                            filter.set('fontcolor', '' + value)
                            letteropacityid.value = getOpacity(value)
                            if (fontRadioButton.checked) mastercolorSwatch.value = value
                        }
                    }
                }
                SliderSpinner {
                    id: letteropacityid
                    minimumValue: 0
                    maximumValue: 100
                    value: getOpacity(fontcolorSwatch.value)
                    decimals: 0
                    stepSize: 1
                    suffix: ' %'
                    Keys.onEnterPressed: gridposition.forceActiveFocus()
                    Keys.onReturnPressed: gridposition.forceActiveFocus()
                    onValueChanged: {
                        blockUpdate = true
                        fontcolorSwatch.value = setOpacity(fontcolorSwatch.value, value)
                        blockUpdate = false
                    }
                }
                UndoButton {
                    onClicked: {
                        letteropacityid.value = 100
                    }
                }
            }
            Item { width: 20 } // Fin Couleur Lettres
            
            Label {
                text: qsTr('Outline')
                Layout.alignment: Qt.AlignRight
            }
            RowLayout {
                Layout.columnSpan: 4
                RadioButton {
                    id: bordsRadioButton
                    exclusiveGroup: colorGroup
                    onClicked: mastercolorSwatch.value = bordscolorSwatch.value
                }
                ColorPicker {
                    id: bordscolorSwatch
                    alpha: true
                    eyedropper: false
                    property bool isReady: false
                    Component.onCompleted: isReady = true
                    onValueChanged: {
                        if (isReady) {
                            filter.set('bordscolor', '' + value)
                            bordscoloropacityid.value = getOpacity(value)
                            if (bordsRadioButton.checked) mastercolorSwatch.value = value
                        }
                    }
                }
                SliderSpinner {
                    id: bordscoloropacityid
                    minimumValue: 0
                    maximumValue: 100
                    value: getOpacity(bordscolorSwatch.value)
                    decimals: 0
                    stepSize: 1
                    suffix: ' %'
                    Keys.onEnterPressed: gridposition.forceActiveFocus()
                    Keys.onReturnPressed: gridposition.forceActiveFocus()
                    onValueChanged: {
                        blockUpdate = true
                        bordscolorSwatch.value = setOpacity(bordscolorSwatch.value, value)
                        blockUpdate = false
                    }
                }
                UndoButton {
                    onClicked: {
                        bordscoloropacityid.value = 100
                    }
                }
            }
            Item { width: 20 } // Fin Couleur Contour
            
            Label {
                text: qsTr('Shadow')
                Layout.alignment: Qt.AlignRight
            }
            RowLayout {
                Layout.columnSpan: 4
                RadioButton {
                    id: shadowRadioButton
                    exclusiveGroup: colorGroup
                    onClicked: mastercolorSwatch.value = shadowcolorSwatch.value
                }
                ColorPicker {
                    id: shadowcolorSwatch
                    alpha: true
                    eyedropper: false
                    property bool isReady: false
                    Component.onCompleted: isReady = true
                    onValueChanged: {
                        if (isReady) {
                            filter.set('shadowcolor', '' + value)
                            shadowopacityid.value = getOpacity(value)
                            if (shadowRadioButton.checked) mastercolorSwatch.value = value
                        }
                    }
                }
                SliderSpinner {
                    id: shadowopacityid
                    minimumValue: 0
                    maximumValue: 100
                    value: getOpacity(shadowcolorSwatch.value)
                    decimals: 0
                    stepSize: 1
                    suffix: ' %'
                    Keys.onEnterPressed: gridposition.forceActiveFocus()
                    Keys.onReturnPressed: gridposition.forceActiveFocus()
                    onValueChanged: {
                        blockUpdate = true
                        shadowcolorSwatch.value = setOpacity(shadowcolorSwatch.value, value)
                        blockUpdate = false
                    }
                }
                UndoButton {
                    onClicked: {
                        shadowopacityid.value = 100
                    }
                }
            }
            Item { width: 20 } // Fin Couleur Ombre
            
            Label {
                text: qsTr('Global')
                Layout.alignment: Qt.AlignRight
            }
            RowLayout {
                Layout.columnSpan: 4
                Label {
                    text: qsTr('text')
                }
                Item { width: 1 }
                SliderSpinner {
                    id: fontopacityid
                    minimumValue: 0
                    maximumValue: 100
                    decimals: 0
                    stepSize: 1
                    suffix: ' %'
                    Keys.onEnterPressed: gridposition.forceActiveFocus()
                    Keys.onReturnPressed: gridposition.forceActiveFocus()
                    onValueChanged: {
                        updateFilterFontopacity(getPosition())
                    }
                }
                UndoButton {
                    onClicked: {
                        fontopacityid.value = 100
                        updateFilterFontopacity(getPosition())
                    }
                }
            }
            KeyframesButton {
                id: fontopacityKeyframesButton
                checked: filter.animateIn <= 0 && filter.animateOut <= 0 && filter.keyframeCount('fontopacity') > 0
                onToggled: {
                    var value = fontopacityid.value
                    if (checked) {
                        filter.clearSimpleAnimation('fontopacity')
                        filter.set('fontopacity', value, getPosition())
                    }
                    else {
                        filter.resetProperty('fontopacity')
                        filter.set('fontopacity', value)
                    }
                    checked = filter.keyframeCount('fontopacity') > 0 && filter.animateIn <= 0 && filter.animateOut <= 0
                }
            } // Fin Opacité Globale
            
        } // Onglet Texte / Police
        
        GridLayout {
            id: gridposition
            Layout.columnSpan: 2
            columns: 3
            anchors.fill: parent
            anchors.leftMargin: 20
            anchors.topMargin: 70
            visible: tabview.currentIndex == 1
        
            Label {
                text: qsTr('Position')
                Layout.alignment: Qt.AlignRight
            } // Position
            RowLayout {
                Layout.columnSpan: 1
                Label { 
                    Layout.preferredWidth : 30
                    horizontalAlignment: Text.AlignRight
                    text: 'X' 
                }
                SpinBox {
                    id: rectX
                    Layout.minimumWidth: 90
                    minimumValue: - profile.width * 2.05
                    maximumValue: profile.width * 1.01
                    decimals: 0
                    stepSize: (booster.checked)? 10 : 1
                    Keys.onEnterPressed: focus = false
                    Keys.onReturnPressed: focus = false
                    onValueChanged: {
                        if ((hovered || activeFocus) && Math.abs(filterRect.x - value) >= 1) {
                            filterRect.x = value
                            updateFilterRect(getPosition(), true)
                        }
                    }
                }
                Label { 
                    Layout.preferredWidth : 30
                    horizontalAlignment: Text.AlignRight
                    text: 'Y' 
                }
                SpinBox {
                    id: rectY
                    Layout.minimumWidth: 90
                    minimumValue: - profile.height * 2.05
                    maximumValue: profile.height * 1.01
                    decimals: 0
                    stepSize: (booster.checked)? 10 : 1
                    Keys.onEnterPressed: focus = false
                    Keys.onReturnPressed: focus = false
                    onValueChanged: {
                        if ((hovered || activeFocus) && Math.abs(filterRect.y - value) >= 1) {
                            filterRect.y = value
                            updateFilterRect(getPosition(), true)
                        }
                    }
                }
                CheckBox {
                    id: booster
                    implicitWidth: 26
                    tooltip: qsTr('Spinboxes steps: 1x or 10x')
                    onClicked: {
                        if (checked) filter.set('_booster', '0')
                        else filter.set('_booster', '1')
                    }
                    style: CheckBoxStyle {
                        indicator: Rectangle {
                            implicitWidth: 26
                            implicitHeight: 20
                            radius: 3
                            color: control.checked? activePalette.highlight : activePalette.button
                            border.color: activePalette.shadow
                            border.width: 1
                            Text {
                                anchors.fill: parent
                                horizontalAlignment: Text.AlignHCenter
                                verticalAlignment: Text.AlignVCenter
                                font.pointSize: (application.OS === 'OS X')? 11 : 8
                                font.bold : true
                                color: activePalette.text
                                text: (control.checked)? qsTr('10x') : qsTr('1x')
                            }
                        }
                    }
                }
                UndoButton {
                    onClicked: {
                        rectX.value = rectY.value = 0
                        filterRect.x = filterRect.y = 0
                        updateFilterRect(getPosition(), true)
                    }
                }
            }
            Row {
                Layout.rowSpan: 2
                KeyframesButton {
                    id: positionKeyframesButton
                    checked: filter.keyframeCount(rectProperty) > 0 && filter.animateIn <= 0 && filter.animateOut <= 0
                    onToggled: {
                        if (checked) {
                            filter.blockSignals = true
                            filter.clearSimpleAnimation(rectProperty)
                            filter.blockSignals = false
                            filter.set(rectProperty, filterRect, 1.0, getPosition())
                        } 
                        else {
                            filter.resetProperty(rectProperty)
                            filter.set(rectProperty, filterRect)
                        }
                        checked = filter.keyframeCount(rectProperty) > 0 && filter.animateIn <= 0 && filter.animateOut <= 0
                    }
                }
            }
                    
            Label {
                text: qsTr('Size')
                Layout.alignment: Qt.AlignRight
            } // Dimensions
            RowLayout {
                Layout.columnSpan: 1
                Label { 
                    Layout.preferredWidth : 30
                    horizontalAlignment: Text.AlignRight
                    text: 'W' 
                }
                SpinBox {
                    id: rectW
                    Layout.minimumWidth: 90
                    minimumValue: 0//contour.value
                    maximumValue: profile.width * 3
                    decimals: 0
                    stepSize: (booster.checked)? 10 : 1
                    Keys.onEnterPressed: focus = false
                    Keys.onReturnPressed: focus = false
                    onValueChanged: {
                        if ((hovered || activeFocus) && Math.abs(filterRect.width - value) >= 1) {
                            var centerX = filterRect.x + filterRect.width / 2
                            var rightX = filterRect.x + filterRect.width
                            filterRect.width = value
                            if (centerRadioButton.checked) {
                                filterRect.x = rectX.value = centerX - filterRect.width / 2
                            } else if (rightRadioButton.checked) {
                                filterRect.x = rectX.value = rightX - filterRect.width
                            }
                            if (ratioCheckbox.checked) {
                                var middleY = filterRect.y + filterRect.height / 2
                                var bottomY = filterRect.y + filterRect.height
                                filterRect.height = rectH.value = Math.round(value / filter.get('_ratio'))
                                if (middleRadioButton.checked) {
                                    filterRect.y = rectY.value = middleY - filterRect.height / 2
                                }
                                else if (bottomRadioButton.checked) {
                                    filterRect.y = rectY.value = bottomY - filterRect.height
                                }
                            }
                            updateFilterRect(getPosition(), true)
                        }
                        else if (!(hovered || rectH.hovered)) {
                            ratioCheckbox.checked = false
                        }
                    }
                }
                Label { 
                    Layout.preferredWidth : 30
                    horizontalAlignment: Text.AlignRight
                    text: 'H' 
                }
                SpinBox {
                    id: rectH
                    Layout.minimumWidth: 90
                    minimumValue: 0//contour.value
                    maximumValue: profile.height * 3
                    decimals: 0
                    stepSize: (booster.checked)? 10 : 1
                    Keys.onEnterPressed: focus = false
                    Keys.onReturnPressed: focus = false
                    onValueChanged: {
                        if ((hovered || activeFocus) && Math.abs(filterRect.height - value) >= 1) {
                            var middleY = filterRect.y + filterRect.height / 2
                            var bottomY = filterRect.y + filterRect.height
                            filterRect.height = value
                            if (middleRadioButton.checked) {
                                filterRect.y = rectY.value = middleY - filterRect.height / 2
                            } else if (bottomRadioButton.checked) {
                                filterRect.y = rectY.value = bottomY - filterRect.height
                            }
                        
                            if (ratioCheckbox.checked) {
                                var centerX = filterRect.x + filterRect.width / 2
                                var rightX = filterRect.x + filterRect.width
                                filterRect.width = rectW.value = Math.round(value * filter.get('_ratio'))
                                if (centerRadioButton.checked) {
                                    filterRect.x = rectX.value = centerX - filterRect.width / 2
                                }
                                else if (rightRadioButton.checked) {
                                    filterRect.x = rectX.value = rightX - filterRect.width
                                }
                            }
                        
                            updateFilterRect(getPosition(), true)
                        }
                        else if (!(hovered || rectW.hovered)) {
                            ratioCheckbox.checked = false
                        }
                    }
                }
                ToggleButton {
                    id: ratioCheckbox
                    implicitWidth: 26
                    implicitHeight: 20
                    Layout.alignment: Qt.AlignCenter
                    iconName: (checked)? 'object-locked' : 'object-unlocked'
                    iconSource: (checked)? 'qrc:///icons/oxygen/32x32/actions/object-locked.png' : 'qrc:///icons/oxygen/32x32/actions/object-unlocked.png'
                    tooltip: qsTr('Lock the current ratio')
                    onClicked: {
                        if (checked) {
                            filter.set('_ratio', filterRect.width / filterRect.height)
                        }
                        else filter.set('_ratio', 0)
                    }
                }
                UndoButton {
                    onClicked: {
                        filterRect.width = rectW.value = profile.width
                        filterRect.height = rectH.value = profile.height
                        updateFilterRect(getPosition(), true)
                    }
                }
            }
            
            Label {
                text: qsTr('Alignment')
                Layout.alignment: Qt.AlignRight
            } // Alignement
            RowLayout {
                Layout.columnSpan: 2
                Item { width: 20 }
                RadioButton {
                    id: leftRadioButton
                    text: 'L'
                    tooltip : 'Left'
                    exclusiveGroup: halignGroup
                    onClicked: if (checked) filter.set('_halign', 1)
                }
                RadioButton {
                    id: centerRadioButton
                    text: 'C'
                    tooltip : 'Center'
                    exclusiveGroup: halignGroup
                    onClicked: if (checked) filter.set('_halign', 2)
                }
                RadioButton {
                    id: rightRadioButton
                    text: 'R'
                    tooltip : 'Right'
                    exclusiveGroup: halignGroup
                    onClicked: if (checked) filter.set('_halign', 3)
                }
                Label {
                    text: '  -  '
                }
                RadioButton {
                    id: topRadioButton
                    text: 'T'
                    tooltip : 'Top'
                    exclusiveGroup: valignGroup
                    onClicked: if (checked) filter.set('_valign', 1)
                }
                RadioButton {
                    id: middleRadioButton
                    text: 'M'
                    tooltip : 'Middle'
                    exclusiveGroup: valignGroup
                    onClicked: if (checked) filter.set('_valign', 2)
                }
                RadioButton {
                    id: bottomRadioButton
                    text: 'B'
                    tooltip : 'Bottom'
                    exclusiveGroup: valignGroup
                    onClicked: if (checked) filter.set('_valign', 3)
                }
            }
            
            Item { 
                Layout.columnSpan: 3
                height : 0
                Layout.fillWidth: true
            } // Double l'interligne
            
            Label {
                text: qsTr('Rotation')
                Layout.alignment: Qt.AlignRight
            } // Rotation
            RowLayout {
                Layout.columnSpan: 1
                Item { width: 10 }
                SliderSpinner {
                    id: rotateslider
                    Layout.maximumWidth: 272
                    minimumValue: -360
                    maximumValue: 360
                    decimals: 1
                    stepSize: 0.1
                    suffix: ' °'
                    Keys.onEnterPressed: gridposition.forceActiveFocus()
                    Keys.onReturnPressed: gridposition.forceActiveFocus()
                    onValueChanged: updateFilterRotate(getPosition())
                }
                UndoButton {
                    onClicked: {
                        rotateslider.value = 0
                        updateFilterRotate(getPosition())
                    }
                }
            }
            KeyframesButton {
                    id: rotateKeyframesButton
                    checked: filter.animateIn <= 0 && filter.animateOut <= 0 && filter.keyframeCount('rotate') > 0
                    onToggled: {
                        var value = rotateslider.value
                        if (checked) {
                            filter.clearSimpleAnimation('rotate')
                            filter.set('rotate', value, getPosition())
                        }
                        else {
                            filter.resetProperty('rotate')
                            filter.set('rotate', value)
                        }
                        checked = filter.keyframeCount('rotate') > 0 && filter.animateIn <= 0 && filter.animateOut <= 0
                    }
                }
            
            Item { 
                Layout.columnSpan: 3
                height : 0
                Layout.fillWidth: true
            } // Double l'interligne
            
            Rectangle {
                Layout.alignment: Qt.AlignRight
                width: 115
                height: 1
                color: "gray"
            }
            RowLayout {
                Layout.columnSpan: 2
                spacing: 5
                Label {
                    text: 'Frame borders'
                    //Layout.alignment: Qt.AlignRight
                }
                Rectangle {
                    color: "gray"
                    Layout.fillWidth: true
                    height: 1
                }
            } // Titre Bordure du cadre
            
            Label { 
                Layout.alignment: Qt.AlignRight
                text: "Sides displayed"
            } // Choix cotés
            RowLayout {
                Layout.columnSpan: 2
                Item { width: 10 }
                ComboBox {
                    id: nombreid
                    Layout.preferredWidth: 250
                    model: ['No side', 'Top', 'Left', 'Top + Left', 'Bottom', 'Top + Bottom', 'Left + Bottom', 'Top + Left + Bottom', 'Right', 'Top + Right', 'Left + Right', 'Left + Top + Right', 'Bottom + Right', 'Top + Right + Bottom', 'Left + Bottom + Right', 'All 4 sides', 'Bottom (tapered ends)']
                    onActivated: {
                        if (index >= 0) {
                            filter.set('number', index)
                        }
                    }
                }
            }
            
            Label { 
                Layout.alignment: Qt.AlignRight
                text: 'Thickness' 
            } // Epaisseur Bordure
            RowLayout {
                Layout.columnSpan: 2
                Item { width: 10 }
                Row {
                    spacing: 5
                    enabled: nombreid.value != 0
                SpinBox {
                    id: contour
                    //enabled: nombreid.value != 0
                    implicitWidth: 90
                    decimals: 0
                    minimumValue: 0
                    maximumValue: 200
                    value: 1
                    suffix: ' px'
                    Keys.onEnterPressed: focus = false
                    Keys.onReturnPressed: focus = false
                    onValueChanged: updateFilterBorder(getPosition())
                    ToolTip { text: 'Thickness of the Frame border' }
                }
                UndoButton {
                    visible: contour.enabled
                    onClicked: {
                        contour.value = 0
                        updateFilterBorder(getPosition())
                    }
                }
                KeyframesButton {
                    id: borderKeyframesButton
                    visible: contour.enabled
                    checked: filter.animateIn <= 0 && filter.animateOut <= 0 && filter.keyframeCount('border') > 0
                    onToggled: {
                        var value = contour.value
                        if (checked) {
                            filter.clearSimpleAnimation('border')
                            filter.set('border', value, getPosition())
                        } else {
                            filter.resetProperty('border')
                            filter.set('border', value)
                        }
                        checked = filter.keyframeCount('border') > 0 && filter.animateIn <= 0 && filter.animateOut <= 0
                    }
                }
                ToggleButton {
                    id: proportionid
                    visible: contour.enabled
                    implicitWidth: 26
                    implicitHeight: 20
                    Layout.alignment: Qt.AlignCenter
                    iconName: 'insert-link'
                    iconSource: 'qrc:///icons/oxygen/32x32/actions/insert-link.png'
                    tooltip: qsTr('If checked: Thickness proportional to the height of the frame')
                    onClicked: {
                        if (checked) {
                            contour.suffix = ' %'
                            filter.set('propor', 1)
                        } else {
                            contour.suffix = ' px'
                            filter.set('propor', 0)
                        }
                    }
                }
                }
            }
            
            Label { 
                Layout.alignment: Qt.AlignRight
                text: "Corner radius"
            } // Rayon d'angle
            RowLayout {
                Layout.columnSpan: 2
                Item { width: 10 }
                Row {
                    spacing: 5
                    enabled: nombreid.currentIndex == 15
                    SpinBox { 
                        id: radiusid
                        implicitWidth: 90
                        minimumValue: 0
                        maximumValue: 100
                        decimals: 0
                        suffix: '  %'
                        Keys.onEnterPressed: focus = false
                        Keys.onReturnPressed: focus = false
                        onValueChanged: updateFilterRadius(getPosition())
                        ToolTip { text: 'Radius of the Frame corners' }
                    }
                    UndoButton {
                        visible: parent.enabled
                        onClicked: {
                            radiusid.value = 0
                            updateFilterRadius(getPosition())
                        }
                    }
                    KeyframesButton {
                        id: radiusKeyframesButton
                        enabled: radiusid.enabled
                        checked: filter.animateIn <= 0 && filter.animateOut <= 0 && filter.keyframeCount('radius') > 0
                        onToggled: {
                            var value = radiusid.value / 100.0
                            if (checked) {
                                filter.clearSimpleAnimation('radius')
                                filter.set('radius', value, getPosition())
                            } else {
                                filter.resetProperty('radius')
                                filter.set('radius', value)
                            }
                            checked = filter.keyframeCount('radius') > 0 && filter.animateIn <= 0 && filter.animateOut <= 0
                        }
                    }
                }
            }
            
            Item { 
                Layout.columnSpan: 3
                height : 0
                Layout.fillWidth: true
            } // Double l'interligne
            
            Rectangle {
                Layout.alignment: Qt.AlignRight
                width: 115
                height: 1
                color: "gray"
            }
            RowLayout {
                Layout.columnSpan: 2
                spacing: 5
                Label {
                    text: 'Color and Opacity'
                    //Layout.alignment: Qt.AlignRight
                }
                Rectangle {
                    color: "gray"
                    Layout.fillWidth: true
                    height: 1
                }
            } // Titre Couleurs
            
            Label { 
                Layout.alignment: Qt.AlignRight
                text: 'Frame background'
            } // Fond du Cadre
            RowLayout {
                RadioButton {
                    id: fillRadioButton
                    exclusiveGroup: colorGroup
                    onClicked: mastercolorSwatch.value = fillcolorSwatch.value
                }
                ColorPicker {
                    id: fillcolorSwatch
                    alpha: true
                    eyedropper: false
                    property bool isReady: false
                    Component.onCompleted: isReady = true
                    onValueChanged: {
                        if (isReady && !blockUpdate) {
                            filter.set('fillcolor', setOpacity(value, 100))
                            fillopacityid.value = getOpacity(value)
                            if (fillRadioButton.checked) mastercolorSwatch.value = value
                        }
                    }
                }
                SliderSpinner {
                    id: fillopacityid
                    Layout.maximumWidth: 240
                    minimumValue: 0
                    maximumValue: 100
                    decimals: 0
                    stepSize: 1
                    suffix: ' %'
                    Keys.onEnterPressed: gridposition.forceActiveFocus()
                    Keys.onReturnPressed: gridposition.forceActiveFocus()
                    onValueChanged: {
                        updateFilterFillopacity(getPosition())
                        // Ligne suivante nécessaire pour la mise à jour de la couleur du ColorPicker mais qui semait le bordel (création d'IC). Rajouté "&& !blockUpdate" dans le test du if (ligne 1930) pour le régler.
                        fillcolorSwatch.value = setOpacity(fillcolorSwatch.value, value)
                        if (fillRadioButton.checked) mastercolorSwatch.value = fillcolorSwatch.value
                    }
                }
                UndoButton {
                    onClicked: {
                        fillopacityid.value = 0
                    }
                }
            }
            KeyframesButton {
                id: fillopacityKeyframesButton
                checked: filter.animateIn <= 0 && filter.animateOut <= 0 && filter.keyframeCount('fillopacity') > 0
                onToggled: {
                    var value = fillopacityid.value
                    if (checked) {
                        filter.clearSimpleAnimation('fillopacity')
                        filter.set('fillopacity', value, getPosition())
                    }
                    else {
                        filter.resetProperty('fillopacity')
                        filter.set('fillopacity', value)
                    }
                    checked = filter.keyframeCount('fillopacity') > 0 && filter.animateIn <= 0 && filter.animateOut <= 0
                
                }
            }
            
            
            Label { 
                Layout.alignment: Qt.AlignRight
                text: 'Frame border'
            } // Bordure du Cadre
            RowLayout {
                RadioButton {
                    id: borderRadioButton
                    exclusiveGroup: colorGroup
                    onClicked: mastercolorSwatch.value = bordercolorSwatch.value
                }
                ColorPicker {
                    id: bordercolorSwatch
                    alpha: false
                    eyedropper: false
                    property bool isReady: false
                    Component.onCompleted: isReady = true
                    onValueChanged: {
                        if (isReady && !blockUpdate) {
                            filter.set('bordercolor', setOpacity(value, 100))
                            borderopacityid.value = getOpacity(value)
                            if (borderRadioButton.checked) mastercolorSwatch.value = value
                        }
                    }
                }
                SliderSpinner {
                    id: borderopacityid
                    Layout.maximumWidth: 240
                    minimumValue: 0
                    maximumValue: 100
                    decimals: 0
                    stepSize: 1
                    suffix: ' %'
                    Keys.onEnterPressed: gridposition.forceActiveFocus()
                    Keys.onReturnPressed: gridposition.forceActiveFocus()
                    onValueChanged: {
                        updateFilterBorderopacity(getPosition())
                        // idem que fillcolor pour le pb des IC
                        bordercolorSwatch.value = setOpacity(bordercolorSwatch.value, value)
                        if (borderRadioButton.checked) mastercolorSwatch.value = bordercolorSwatch.value
                    }
                }
                UndoButton {
                    onClicked: {
                        borderopacityid.value = 100
                    }
                }
            }
            KeyframesButton {
                id: borderopacityKeyframesButton
                checked: filter.animateIn <= 0 && filter.animateOut <= 0 && filter.keyframeCount('borderopacity') > 0
                onToggled: {
                    var value = borderopacityid.value
                    if (checked) {
                        filter.clearSimpleAnimation('borderopacity')
                        filter.set('borderopacity', value, getPosition())
                    }
                    else {
                        filter.resetProperty('borderopacity')
                        filter.set('borderopacity', value)
                    }
                    checked = filter.keyframeCount('borderopacity') > 0 && filter.animateIn <= 0 && filter.animateOut <= 0
                }
            }
        
        } // Onglet Encadrement
        
        GridLayout {
            id: gridcadre
            Layout.columnSpan: 2
            columns: 3
            anchors.fill: parent
            anchors.leftMargin: 20
            anchors.topMargin: 80
            visible: tabview.currentIndex == 2
        
            Label { 
                Layout.alignment: Qt.AlignRight
                text: 'Alignment' 
            } // Alignement
            RowLayout {
                Layout.columnSpan: 2
                Item { width: 10 }
                ToggleButton {
                    id: textalignleftid
                    exclusiveGroup: textAlignGroup
                    implicitWidth: 26
                    implicitHeight: 20
                    iconName: 'format-justify-left.png'
                    iconSource: 'qrc:///icons/oxygen/32x32/actions/format-justify-left.png'
                    tooltip: qsTr('Left')
                    onClicked: {
                        if (checked) {
                            filter.set('textalign', 'left')
                        }
                    }
                }
                ToggleButton {
                    id: textaligncenterid
                    exclusiveGroup: textAlignGroup
                    implicitWidth: 26
                    implicitHeight: 20
                    iconName: 'format-justify-center.png'
                    iconSource: 'qrc:///icons/oxygen/32x32/actions/format-justify-center.png'
                    tooltip: qsTr('Center')
                    onClicked: {
                        if (checked) filter.set('textalign', 'center')
                    }
                }
                ToggleButton {
                    id: textalignrightid
                    exclusiveGroup: textAlignGroup
                    implicitWidth: 26
                    implicitHeight: 20
                    iconName: 'format-justify-right.png'
                    iconSource: 'qrc:///icons/oxygen/32x32/actions/format-justify-right.png'
                    tooltip: qsTr('Right')
                    onClicked: {
                        if (checked) {
                            filter.set('textalign', 'right')
                        }
                    }
                }
            }
        
            Label { 
                Layout.alignment: Qt.AlignRight
                text: {
                    (textalignleftid.checked)? 'Left margin' : 
                    (textalignrightid.checked)? 'Right margin' :
                    (textajustid.checked && modeStdRadioButton.checked)? 'L & R margins' : 'No margins'
                }
            } // Marges
            RowLayout {
                Layout.columnSpan: 2
                enabled: textalignleftid.checked || textalignrightid.checked || (textajustid.checked && modeStdRadioButton.checked)
                Item { width: 10 }
                SpinBox { 
                    id: marginid
                    Layout.preferredWidth: 75
                    minimumValue: -profile.width
                    maximumValue: profile.width
                    decimals: 0
                    suffix: qsTr(' px')
                    Keys.onEnterPressed: focus = false
                    Keys.onReturnPressed: focus = false
                    onValueChanged: updateFilterMargin(getPosition())
                    ToolTip {
                        text: 'Sets the width of the margin(s)\non the left and/or right side(s)\nof the text in the Frame'
                    }
                }
                UndoButton {
                    visible: textalignleftid.checked || textalignrightid.checked || (textajustid.checked && modeStdRadioButton.checked)
                    onClicked: {
                        marginid.value = 0
                        updateFilterMargin(getPosition())
                    }
                }
                KeyframesButton {
                    id: marginKeyframesButton
                    checked: filter.animateIn <= 0 && filter.animateOut <= 0 && filter.keyframeCount('margin') > 0
                    onToggled: {
                        var value = marginid.value
                        if (checked) {
                            filter.clearSimpleAnimation('margin')
                            filter.set('margin', value, getPosition())
                        } else {
                            filter.resetProperty('margin')
                            filter.set('margin', value)
                        }
                        checked = filter.keyframeCount('margin') > 0 && filter.animateIn <= 0 && filter.animateOut <= 0
                    }
                }
            }
        
            Label {
                Layout.alignment: Qt.AlignRight
                text: "Constrain text"
            } // Texte Contraint
            RowLayout {
                Layout.columnSpan: 2
                enabled: modeStdRadioButton.checked
                Item { width: 10 }
                ToggleButton {
                    id: textajustid
                    opacity: modeStdRadioButton.checked? 1 : 0.5
                    implicitWidth: 26
                    implicitHeight: 20
                    iconName: 'scrub_drag'
                    iconSource: 'qrc:///icons/oxygen/32x32/actions/scrub_drag.png'
                    tooltip: qsTr('If checked, the text won\'t exceed the limits of the Backdrop, it will be narrowed.')
                    onClicked: {
                        if (checked) filter.set('textajust', 1)
                        else filter.set('textajust', 0)
                    }
                }
                Label {
                    visible: modeTitreRadioButton.checked
                    opacity: 0.6
                    text: " (In Standard Mode only)"
                }
            }
        
            Label {
                Layout.alignment: Qt.AlignRight
                text: "Text width"
            } // Largeur du texte
            RowLayout {
                Layout.columnSpan: 2
                enabled: modeTitreRadioButton.checked
                Item { width: 10 }
                SpinBox { 
                    id: interlettreid
                    implicitWidth: 75
                    minimumValue: 0 //*****
                    maximumValue: 999 //*****
                    decimals: 0
                    suffix: qsTr(' % ')
                    value: 100
                    Keys.onEnterPressed: focus = false
                    Keys.onReturnPressed: focus = false
                    onValueChanged: updateFilterInterlettre(getPosition())
                    ToolTip { text: 'Adjust the width of the line of text by increasing/decreasing\nthe space between each characters' }
                }
                UndoButton {
                    visible: modeTitreRadioButton.checked
                    onClicked: interlettreid.value = 100
                }
                KeyframesButton {
                    id: interlettreKeyframesButton
                    checked: filter.animateIn <= 0 && filter.animateOut <= 0 && filter.keyframeCount('interlettre') > 0
                    onToggled: {
                        var value = interlettreid.value
                        if (checked) {
                            filter.clearSimpleAnimation('interlettre')
                            filter.set('interlettre', value, getPosition())
                        } else {
                            filter.resetProperty('interlettre')
                            filter.set('interlettre', value)
                        }
                        checked = filter.keyframeCount('interlettre') > 0 && filter.animateIn <= 0 && filter.animateOut <= 0
                    }
                }
                ToggleButton {
                    id: letterwidthid
                    visible: modeTitreRadioButton.checked
                    implicitWidth: 28
                    implicitHeight: 20
                    iconName: 'font'
                    iconSource: 'qrc:///icons/oxygen/32x32/actions/font.png'
                    tooltip: qsTr('When reducing the width at a value inferior than 100%:\nUnchecked: Character & space are reduced\nChecked: Only the space between charaters is reduced.')
                    onClicked: {
                        if (checked) filter.set('letterwidth', 1)
                        else filter.set('letterwidth', 0)
                    }
                }
                Label {
                    visible: modeStdRadioButton.checked
                    opacity: 0.6
                    text: "(In Title Mode only)"
                }
            }
        
            Label { 
                Layout.alignment: Qt.AlignRight
                text: 'Vertical Position' 
                } // Position Verticale
            RowLayout {
                Layout.columnSpan: 2
                Item { width: 10 }
                SpinBox { 
                    id: ajustid
                    Layout.preferredWidth: 75
                    minimumValue: -profile.height
                    maximumValue: profile.height
                    decimals: 0
                    suffix: qsTr(' px')
                    Keys.onEnterPressed: focus = false
                    Keys.onReturnPressed: focus = false
                    onValueChanged: updateFilterAjust(getPosition())
                    ToolTip { text: 'Changes the vertical position of the text in the Backdrop' }
                }
                UndoButton {
                    onClicked: {
                        ajustid.value = 0
                        updateFilterAjust(getPosition())
                    }
                }
                KeyframesButton {
                    id: ajustKeyframesButton
                    checked: filter.animateIn <= 0 && filter.animateOut <= 0 && filter.keyframeCount('ajust') > 0
                    onToggled: {
                        var value = ajustid.value
                        if (checked) {
                            filter.clearSimpleAnimation('ajust')
                            filter.set('ajust', value, getPosition())
                        } else {
                            filter.resetProperty('ajust')
                            filter.set('ajust', value)
                        }
                        checked = filter.keyframeCount('ajust') > 0 && filter.animateIn <= 0 && filter.animateOut <= 0
                    }
                }
            }
        
            Label { 
                Layout.alignment: Qt.AlignRight
                text: "Line spacing" 
            } // Interligne
            RowLayout {
                Layout.columnSpan: 2
                Item { width: 10 }
                SpinBox { 
                    id: interligneid
                    implicitWidth: 75
                    minimumValue: 0
                    maximumValue: 10
                    decimals: 1
                    stepSize: 0.1
                    suffix: qsTr(' x  ')
                    value: 1
                    Keys.onEnterPressed: focus = false
                    Keys.onReturnPressed: focus = false
                    onValueChanged: filter.set('interligne', value)
                    ToolTip { text: 'Changes the spacing between lines when\nthere is more than one line of text)' }
                }
                UndoButton {
                    onClicked: interligneid.value = 1
                }
            }
        
            Label { 
                Layout.alignment: Qt.AlignRight
                text: "Display order" 
            } // Ordre d'affichage
            RowLayout {
                Layout.columnSpan: 2
                Item { width: 10 }
                ComboBox {
                    id: borderoverid
                    implicitWidth: 275
                    model: ['Backdrop borders are UNDER the text', 'Backdrop borders are ABOVE the text']
                    onActivated: {
                        if (index >= 0) {
                            filter.set('borderover', index)
                        }
                    }
                }
                
            }
            
            Item { 
                Layout.columnSpan: 3
                height : 5
                Layout.fillWidth: true
            } // Double l'interligne
            
            Rectangle {
                Layout.alignment: Qt.AlignRight
                width: 115
                height: 1
                color: "gray"
            }
            RowLayout {
                Layout.columnSpan: 2
                Layout.maximumWidth: 340
                spacing: 5
                Label {
                    text: 'Color and Opacity'
                }
                Rectangle {
                    color: "gray"
                    Layout.fillWidth: true
                    height: 1
                }
            } // Titre Couleurs et Opacités
            
            Item { 
                Layout.columnSpan: 3
                height : 0
                Layout.fillWidth: true
            } // Double l'interligne
            
            Label {
                text: qsTr('Background')
                Layout.alignment: Qt.AlignRight
            } // Couleur de l'arrière plan
            RowLayout {
                Layout.columnSpan: 2
                Layout.maximumWidth: 340
                RadioButton {
                    id: bgRadioButton
                    exclusiveGroup: colorGroup
                    onClicked: mastercolorSwatch.value = bgcolorSwatch.value
                }
                ColorPicker {
                    id: bgcolorSwatch
                    alpha: true
                    eyedropper: false
                    property bool isReady: false
                    Component.onCompleted: isReady = true
                    onValueChanged: {
                        if (isReady) {
                            filter.set('color', '' + value)
                            bgopacityid.value = getOpacity(value)
                            if (bgRadioButton.checked) mastercolorSwatch.value = value
                        }
                    }
                }
                SliderSpinner {
                    id: bgopacityid
                    minimumValue: 0
                    maximumValue: 100
                    value: getOpacity(bgcolorSwatch.value)
                    decimals: 0
                    stepSize: 1
                    suffix: ' %'
                    Keys.onEnterPressed: gridposition.forceActiveFocus()
                    Keys.onReturnPressed: gridposition.forceActiveFocus()
                    onValueChanged: {
                        blockUpdate = true
                        bgcolorSwatch.value = setOpacity(bgcolorSwatch.value, value)
                        blockUpdate = false
                    }
                }
                UndoButton {
                    onClicked: {
                        bgopacityid.value = 100
                    }
                }
            }
            
            Item { 
                Layout.columnSpan: 3
                height : 0
                Layout.fillWidth: true
            } // Double l'interligne
            
            Label {
                text: qsTr('Alpha layer of the clip')
                Layout.alignment: Qt.AlignRight
            } // Couche Alpha du clip
            RowLayout {
                Layout.columnSpan: 2
                ToggleButton {
                    id: refclip
                    enabled: bgopacityid.value == 0 //bgcolorSwatch.value == '#00000000'
                    implicitWidth: 70
                    implicitHeight: 20
                    text: (checked)? qsTr('Alpha = 0       ') : qsTr('Alpha = 1       ')
                    tooltip: qsTr('Alpha layer of the clip')
                    onCheckedChanged: {
                        if (checked) filter.set('transparent', 1)
                        else filter.set('transparent', 0)
                    }
                }
                Label {
                    text: "Alpha = 0 makes the clip transparent"
                    opacity: 0.7
                    font.pointSize: font.pointSize - 2
                    font.italic : true
                }
            }
        } // Onglet Mise en forme

        GridLayout {
            id: gridmaster
            Layout.columnSpan: 2
            columns: 3
            anchors.fill: parent
            anchors.leftMargin: 20
            anchors.topMargin: 380
        Label {
            text: qsTr('Color Master')
            Layout.alignment: Qt.AlignRight
        }
        RowLayout {
            Layout.columnSpan: 1
            Item { width: 1 } //Espace
            ColorPicker {
                id: mastercolorSwatch
                alpha: true
                property bool isReady: false
                Component.onCompleted: isReady = true
                ToolTip { text: "The color selected in the Color Master\nis applied to the checked parameter.." }
                onValueChanged: {
                    if (isReady) {
                        //blockUpdate = true
                        if (fillRadioButton.checked) fillcolorSwatch.value = value
                        if (borderRadioButton.checked) bordercolorSwatch.value = value
                        if (bgRadioButton.checked) bgcolorSwatch.value = value
                        if (fontRadioButton.checked) fontcolorSwatch.value = value
                        if (bordsRadioButton.checked) bordscolorSwatch.value = value
                        if (shadowRadioButton.checked) shadowcolorSwatch.value = value
                        //blockUpdate = false
                    }
                }
                onPickStarted: {
                    //filter.set("disable", 1);
                }
                onPickCancelled: filter.set('disable', 0)
            }
            
            Item { width: 10 } //Espace
            
            //****** Série de boutons ronds en couleur ****
            
            Rectangle { //Bouton Transparent
                id: transparentColor
                //visible: bgRadioButton.checked || fontRadioButton.checked || bordsRadioButton.checked || shadowRadioButton.checked
                implicitWidth: 12
                implicitHeight: 12
                color: '#00000000'
                border.width: 1
                border.color: 'gray'
                radius: implicitWidth / 2
                ToolTip {
                    parent: transparentColor
                    text: qsTr('Transparent')
                }
                MouseArea {
                    anchors.fill: parent
                    acceptedButtons: Qt.LeftButton
                    onPressed: mastercolorSwatch.value = setOpacity(mastercolorSwatch.value, 0)
                }
            }
            Item { width: 10 } //Espace
            Rectangle { //Bouton Noir
                id: noirColor
                implicitWidth: 12
                implicitHeight: 12
                color: '#FF000000'
                border.width: 1
                border.color: 'gray'
                radius: implicitWidth / 2
                ToolTip {
                    parent: noirColor
                    text: qsTr('Black')
                }
                MouseArea {
                    anchors.fill: parent
                    acceptedButtons: Qt.LeftButton
                    onPressed: mastercolorSwatch.value = '#FF000000'
                }
            }
            Rectangle { //Bouton Blanc
                id: blancColor
                implicitWidth: 12
                implicitHeight: 12
                color: '#FFFFFFFF'
                border.width: 1
                border.color: 'gray'
                radius: implicitWidth / 2
                ToolTip {
                    parent: blancColor
                    text: qsTr('White')
                }
                MouseArea {
                    anchors.fill: parent
                    acceptedButtons: Qt.LeftButton
                    onPressed: mastercolorSwatch.value = '#FFFFFFFF'
                }
            }
            Rectangle { //Bouton Rouge
                implicitWidth: 12
                implicitHeight: 12
                color: '#FFFF0000'
                border.width: 1
                border.color: 'gray'
                radius: implicitWidth / 2
                MouseArea {
                    anchors.fill: parent
                    acceptedButtons: Qt.LeftButton
                    onPressed: mastercolorSwatch.value = '#FFFF0000'
                }
            }
            Rectangle { //Bouton Orange
                implicitWidth: 12
                implicitHeight: 12
                color: '#FFFF7F00'
                border.width: 1
                border.color: 'gray'
                radius: implicitWidth / 2
                MouseArea {
                    anchors.fill: parent
                    acceptedButtons: Qt.LeftButton
                    onPressed: mastercolorSwatch.value = '#FFFF7F00'
                }
            }
            Rectangle { //Bouton Jaune
                implicitWidth: 12
                implicitHeight: 12
                color: '#FFFFFF00'
                border.width: 1
                border.color: 'gray'
                radius: implicitWidth / 2
                MouseArea {
                    anchors.fill: parent
                    acceptedButtons: Qt.LeftButton
                    onPressed: mastercolorSwatch.value = '#FFFFFF00'
                }
            }
            Rectangle { //Bouton Vert
                implicitWidth: 12
                implicitHeight: 12
                color: '#FF00FF00'
                border.width: 1
                border.color: 'gray'
                radius: implicitWidth / 2
                MouseArea {
                    anchors.fill: parent
                    acceptedButtons: Qt.LeftButton
                    onPressed: mastercolorSwatch.value = '#FF00FF00'
                }
            }
            Rectangle { //Bouton Cyan
                implicitWidth: 12
                implicitHeight: 12
                color: '#FF00FFFF'
                border.width: 1
                border.color: 'gray'
                radius: implicitWidth / 2
                MouseArea {
                    anchors.fill: parent
                    acceptedButtons: Qt.LeftButton
                    onPressed: mastercolorSwatch.value = '#FF00FFFF'
                }
            }
            Rectangle { //Bouton Bleu
                implicitWidth: 12
                implicitHeight: 12
                color: '#FF0000FF'
                border.width: 1
                border.color: 'gray'
                radius: implicitWidth / 2
                MouseArea {
                    anchors.fill: parent
                    acceptedButtons: Qt.LeftButton
                    onPressed: mastercolorSwatch.value = '#FF0000FF'
                }
            }
            Rectangle { //Bouton Violet
                implicitWidth: 12
                implicitHeight: 12
                color: '#FF7F007F'
                border.width: 1
                border.color: 'gray'
                radius: implicitWidth / 2
                MouseArea {
                    anchors.fill: parent
                    acceptedButtons: Qt.LeftButton
                    onPressed: mastercolorSwatch.value = '#FF7F007F'
                }
            }
            Rectangle { //Bouton Magenta
                implicitWidth: 12
                implicitHeight: 12
                color: '#FFFF00FF'
                border.width: 1
                border.color: 'gray'
                radius: implicitWidth / 2
                MouseArea {
                    anchors.fill: parent
                    acceptedButtons: Qt.LeftButton
                    onPressed: mastercolorSwatch.value = '#FFFF00FF'
                }
            }
            Rectangle { //Bouton Brun
                implicitWidth: 12
                implicitHeight: 12
                color: '#FF996633'
                border.width: 1
                border.color: 'gray'
                radius: implicitWidth / 2
                MouseArea {
                    anchors.fill: parent
                    acceptedButtons: Qt.LeftButton
                    onPressed: mastercolorSwatch.value = '#FF996633'
                }
            }
            //****** Fin de la série de boutons ronds en couleur ***
        }
        Item { width: 20 }
        
    } // Onglet Master

    }

    function updateFilter() {
        updateFilterRadius(null)
        updateFilterMargin(null)
        updateFilterInterlettre(null)
        updateFilterAjust(null)
        updateFilterBorder(null)
        updateFilterFillopacity(null)
        updateFilterBorderopacity(null)
        updateFilterFontopacity(null)
        updateFilterFontsize(null)
        updateFilterRotate(null)
        updateFilterRect(null)
    }
    
    Connections {
        target: filter
        onChanged: {
            setRectControls()
            //setRotateControls() // pas sûr nécessaire
        }
        onInChanged: updateFilter()
        onOutChanged: updateFilter()
        onAnimateInChanged: updateFilter()
        onAnimateOutChanged: updateFilter()
    }

    Connections {
        target: producer
        onPositionChanged: {
            setRadiusControls()
            setMarginControls()
            setInterlettreControls()
            setAjustControls()
            setBorderControls()
            setFillopacityControls()
            setBorderopacityControls()
            setFontopacityControls()
            setFontsizeControls()
            setRotateControls()
            setRectControls()
        }
    }
}